### EXAMPLE 2
### MARCH 1785

library(dataresqc)
library(ncdf4)
library(chron)


# calculate anomalies of SMP March 1785 to EKF400v2 reanalysis
setwd("~/Scripts/Case studies/")

ncname <- list.files()[2] #change file index accordingly
dname <- "air_temperature"
ncin <- nc_open(ncname)

Year <- 1701:2000
BUD <- data.frame(Year, round(ncvar_get(ncin,dname),2)) #change file name accordingly

#for MAN & PAD in EKF, coldest March and FMA in 300 years
#for ZAG in EKF, 2nd coldest March (to 1845) and coldest FMA in 300 years
#for BUD in EKF, 9th coldest March and FMA in 300 years

-5.3 - mean(MAN[51:100,2]); -5.3 - mean(MAN[251:300,2]); -5.3 - mean(MAN[81:92,2])

-6 - mean(ZAG[51:100,2]); -6 - mean(ZAG[251:300,2]); -6 - mean(MAN[81:92,2])

-2.6 - mean(PAD[51:100,2]); -2.6 - mean(PAD[251:300,2]); -2.6 - mean(PAD[81:92,2])

-2.2 - mean(BUD[51:100,2]); -2.2 - mean(BUD[251:300,2]); -2.2 - mean(BUD[c(82:86,88:92),2])



# create table of series for Fig. 8

setwd("~/Palatina_corr/7_Daily-series/ta/") #path to folder with ta daily series, should contain 37 SEF files

MAN <- read_sef("PALAEO-RA_Palatine-Society_Mannheim_17810101-17921231_ta_daily.tsv", all=F)
PAD <- read_sef("PALAEO-RA_Palatine-Society_Padua_17810101-17921231_ta_daily.tsv", all=F)
BUD <- read_sef("PALAEO-RA_Palatine-Society_Budapest_17811108-17921231_ta_daily.tsv", all=F)
ZAG <- read_sef("PALAEO-RA_Palatine-Society_Zagan_17810101-17921231_ta_daily.tsv", all=F)
PRA <- read_sef("PALAEO-RA_Palatine-Society_Prague_17810801-17911231_ta_daily.tsv", all=F)
REG <- read_sef("PALAEO-RA_Palatine-Society_Regensburg_17810101-17911231_ta_daily.tsv", all=F)
BRL <- read_sef("PALAEO-RA_Palatine-Society_Berlin_17810701-17881231_ta_daily.tsv", all=F)
GEN <- read_sef("PALAEO-RA_Palatine-Society_Geneva_17820101-17891231_ta_daily.tsv", all=F)
COP <- read_sef("PALAEO-RA_Palatine-Society_Copenhagen_17820101-17881231_ta_daily.tsv", all=F)
HOH <- read_sef("PALAEO-RA_Palatine-Society_Hohenpeissenberg_17810101-17921231_ta_daily.tsv", all=F)
WUR <- read_sef("PALAEO-RA_Palatine-Society_Wuerzburg_17810101-17881231_ta_daily.tsv", all=F)
BRU <- read_sef("PALAEO-RA_Palatine-Society_Brussels_17820101-17921231_ta_daily_qc.tsv", all=F)

Date <- as.character(as.Date(paste0(MAN$Year,"-",MAN$Month,"-",MAN$Day)))
Date <- Date[grepl("85-02|85-03|85-04",Date)]

MAN <- MAN[grepl("85",MAN$Year) & grepl("2|3|4",MAN$Month) & !grepl("12",MAN$Month), 5]
PAD <- PAD[grepl("85",PAD$Year) & grepl("2|3|4",PAD$Month) & !grepl("12",PAD$Month), 5]
BUD <- BUD[grepl("85",BUD$Year) & grepl(".*2|3|4",BUD$Month) & !grepl("12",BUD$Month), 5]
ZAG <- ZAG[grepl("85",ZAG$Year) & grepl(".*2|3|4",ZAG$Month) & !grepl("12",ZAG$Month), 5]
PRA <- PRA[grepl("85",PRA$Year) & grepl(".*2|3|4",PRA$Month) & !grepl("12",PRA$Month), 5]
REG <- REG[grepl("85",REG$Year) & grepl(".*2|3|4",REG$Month) & !grepl("12",REG$Month), 5]
BRL <- BRL[grepl("85",BRL$Year) & grepl(".*2|3|4",BRL$Month) & !grepl("12",BRL$Month), 5]
GEN <- GEN[grepl("85",GEN$Year) & grepl(".*2|3|4",GEN$Month) & !grepl("12",GEN$Month), 5]
COP <- COP[grepl("85",COP$Year) & grepl(".*2|3|4",COP$Month) & !grepl("12",COP$Month), 5]
HOH <- HOH[grepl("85",HOH$Year) & grepl(".*2|3|4",HOH$Month) & !grepl("12",HOH$Month), 5]
WUR <- WUR[grepl("85",WUR$Year) & grepl(".*2|3|4",WUR$Month) & !grepl("12",WUR$Month), 5]
BRU <- BRU[grepl("85",BRU$Year) & grepl(".*2|3|4",BRU$Month) & !grepl("12",BRU$Month), 5]

march85 <- cbind(MAN, PAD, BUD, ZAG, PRA, REG, BRL, GEN, COP, HOH, WUR, BRU)

write.table(data.frame(Date,march85),"../Code+Additional/Case studies/march85ta.txt",sep="\t",row.names=F,col.names=T) #insert full path
