### EXAMPLE 1
### STORMS 83/84

library(dataresqc)
library(corrr)

# read files that have data Dec 83 to Jan 84

codes <- c("AND", "BER", "BOL", "BRU", "BUD", "CAM", "COP", "DEL", "HAG", "DIJ", "DUS", "EDS",
           "ERF", "GEN", "GDH", "GTN", "GOT", "HOH", "ING", "ROC", "MAN", "MAR", "MID", "MOS",
           "MUN", "PAD", "PRA", "PYS", "REG", "ROM", "SPY", "STO", "PET", "ZEN", "TEG", "WUR", "ZAG")

for (i in 1:length(codes)) {
    setwd("../../Palatina corr/7_daily-series/p/") #path to folder with p daily series, should contain 37 SEF files
    files <- list.files(pattern=".tsv")
    cod <- paste0(codes[i], "_p")
    sef <- read_sef(files[i], all=F)
    sef$Date <- as.Date(paste0(sef$Year,"-",sef$Month,"-",sef$Day))
    sef <- sef[grepl("1783-12|1784-01", sef$Date),]
    assign(cod, sef[,5:6])
    setwd("../../Palatina corr/7_daily-series/ta/") #path to folder with ta daily series, should contain 37 SEF files
    files <- list.files(pattern=".tsv")
    cod <- paste0(codes[i], "_ta")
    sef <- read_sef(files[i], all=F)
    sef$Date <- as.Date(paste0(sef$Year,"-",sef$Month,"-",sef$Day))
    sef <- sef[grepl("1783-12|1784-01", sef$Date),]
    assign(cod, sef[,5:6])
}
rm(list=ls(pattern="BOL|CAM|DEL|DUS|EDS|GDH|HAG|ING|MOS|PYS|ZEN"))


p <- list(AND_p,BER_p,BRU_p,BUD_p,COP_p,DIJ_p,ERF_p,GEN_p,GOT_p,GTN_p,
          HOH_p,MAN_p,MAR_p,MID_p,MUN_p,PAD_p,PET_p,PRA_p,REG_p,ROC_p,
          ROM_p,SPY_p,STO_p,TEG_p,WUR_p,ZAG_p)
ta <- list(AND_ta,BER_ta,BRU_ta,BUD_ta,COP_ta,DIJ_ta,ERF_ta,GEN_ta,GOT_ta,GTN_ta,
           HOH_ta,MAN_ta,MAR_ta,MID_ta,MUN_ta,PAD_ta,PET_ta,PRA_ta,REG_ta,ROC_ta,
           ROM_ta,SPY_ta,STO_ta,TEG_ta,WUR_ta,ZAG_ta)


# reduction to sl
codes <- sort(codes[!grepl("BOL|CAM|DEL|DUS|EDS|GDH|HAG|ING|MOS|PYS|ZEN",codes)])
alt <- c(710,45,60,168,40,250,202,380,2093,150,977,112,44,15,525,18,15,200,346,19,56,110,44,740,200,116)
R <- 287.05
a <- 6.5*(10^-3)
g <- 9.8
for (i in 1:length(p)) {
  x <- (g/R) * alt[i]
  y <- (273.15 + ta[[i]][1]) + (a * (alt[i]/2))
  z <- x / y
  slp <- round(p[[i]][1] * exp(z), 1)
  cod <- paste0(codes[i], "_slp")
  assign(cod, slp)
}


# mutual correlations
SPY_slp <- data.frame(c(SPY_slp[1:54,], NA, SPY_slp[55:61,]))
slp <- cbind(AND_slp,BER_slp,BRU_slp,BUD_slp,COP_slp,DIJ_slp,ERF_slp,GEN_slp,GOT_slp,GTN_slp,
           HOH_slp,MAN_slp,MAR_slp,MID_slp,MUN_slp,PAD_slp,PET_slp,PRA_slp,REG_slp,ROC_slp,
           ROM_slp,SPY_slp,STO_slp,TEG_slp,WUR_slp,ZAG_slp)
colnames(slp) <- codes
cor <- correlate(slp, method = "pearson")


# write table
Date <- as.character(sef$Date)
write.table(data.frame(Date,slp),"../../Scripts/Case studies/storms83-4.txt",sep="\t",row.names=F,col.names=T) #insert full path


### EXAMPLE 2
### MARCH 85

# read files that have data Mar 85

codes <- c("AND", "BER", "BOL", "BRU", "BUD", "CAM", "COP", "DEL", "HAG", "DIJ", "DUS", "EDS",
           "ERF", "GEN", "GDH", "GTN", "GOT", "HOH", "ING", "ROC", "MAN", "MAR", "MID", "MOS",
           "MUN", "PAD", "PRA", "PYS", "REG", "ROM", "SPY", "STO", "PET", "ZEN", "TEG", "WUR", "ZAG")

for (i in 1:length(codes)) {
  setwd("") #path to folder with p daily series, should contain 37 SEF files
  files <- list.files(pattern=".tsv")
  cod <- paste0(codes[i], "_p")
  sef <- read_sef(files[i], all=F)
  sef$Date <- as.Date(paste0(sef$Year,"-",sef$Month,"-",sef$Day))
  sef <- sef[grepl("1785-03", sef$Date),]
  assign(cod, sef[,5:6])
  setwd("") #path to folder with ta daily series, should contain 37 SEF files
  files <- list.files(pattern=".tsv")
  cod <- paste0(codes[i], "_ta")
  sef <- read_sef(files[i], all=F)
  sef$Date <- as.Date(paste0(sef$Year,"-",sef$Month,"-",sef$Day))
  sef <- sef[grepl("1785-03", sef$Date),]
  assign(cod, sef[,5:6])
}
rm(list=ls(pattern="BOL|CAM|DEL|DIJ|DUS|EDS|GDH|HAG|ING|MOS|PYS|ZEN"))


p <- list(AND_p,BER_p,BRU_p,BUD_p,COP_p,ERF_p,GEN_p,GOT_p,GTN_p,
          HOH_p,MAN_p,MAR_p,MID_p,MUN_p,PAD_p,PET_p,PRA_p,REG_p,ROC_p,
          ROM_p,SPY_p,STO_p,TEG_p,WUR_p,ZAG_p)
ta <- list(AND_ta,BER_ta,BRU_ta,BUD_ta,COP_ta,ERF_ta,GEN_ta,GOT_ta,GTN_ta,
           HOH_ta,MAN_ta,MAR_ta,MID_ta,MUN_ta,PAD_ta,PET_ta,PRA_ta,REG_ta,ROC_ta,
           ROM_ta,SPY_ta,STO_ta,TEG_ta,WUR_ta,ZAG_ta)


# reduction to sl
codes <- sort(codes[!grepl("BOL|CAM|DEL|DIJ|DUS|EDS|GDH|HAG|ING|MOS|PYS|ZEN",codes)])
alt <- c(710,45,60,168,40,202,380,2093,150,977,112,44,15,525,18,15,200,346,19,56,110,44,740,200,116)
R <- 287.05
a <- 6.5*(10^-3)
g <- 9.8
for (i in 1:length(p)) {
  x <- (g/R) * alt[i]
  y <- (273.15 + ta[[i]][1]) + (a * (alt[i]/2))
  z <- x / y
  slp <- round(p[[i]][1] * exp(z), 1)
  cod <- paste0(codes[i], "_slp")
  assign(cod, slp)
}


# mutual correlations
slp <- cbind(AND_slp,BER_slp,BRU_slp,BUD_slp,COP_slp,ERF_slp,GEN_slp,GOT_slp,GTN_slp,
             HOH_slp,MAN_slp,MAR_slp,MID_slp,MUN_slp,PAD_slp,PET_slp,PRA_slp,REG_slp,ROC_slp,
             ROM_slp,SPY_slp,STO_slp,TEG_slp,WUR_slp,ZAG_slp)
colnames(slp) <- codes
cor <- correlate(slp, method = "pearson")


# write table
Date <- as.character(sef$Date)
write.table(data.frame(Date,slp),"../../Scripts/Case studies/march85slp.txt",sep="\t",row.names=F,col.names=T)
