## Calculate monthly means from daily means
##
## Monthly means are not calculated if either of the following criteria are satisfied (WMO, 2011):
## - Observations are missing for 11 or more days
## - Observations are missing for a period of 5 or more consecutive days


library(dataresqc)
library(lubridate)

inpath <- "../../5_QCed/Rome/"
#inpath <- "../../6_ERA5-means/Rome/"

outpath <- "../../6_ERA5-means/Rome/"

count_days <- function(x) {
  n <- length(x)
  if (n >= 18) {
    x <- c(0, x)
    fdiff <- x[2:(n+1)]-x[1:n]
    if (max(fdiff) > 5) n <- 0
  }
  return(n)
}

## Temperature and pressure
files <- list.files(inpath, pattern="daily", full.names=TRUE)
for (f in files) {
  x <- read_sef(f)
  x <- x[which(!is.na(x$Value)), ]
  meta <- read_meta(f)
  xmonthly <- aggregate(x$Value, list(x$Year,x$Month), mean)
  xmonthly <- xmonthly[order(xmonthly[,1],xmonthly[,2]),]
  nmonthly <- aggregate(x$Day, list(x$Year,x$Month), count_days)
  nmonthly <- nmonthly[order(nmonthly[,1],nmonthly[,2]),]
  nmonthly$miss <- days_in_month(as.Date(paste(nmonthly[,1],nmonthly[,2],1,sep="-"))) -
    nmonthly$x
  xmonthly$x[which(nmonthly$miss>=11)] <- NA
  out <- data.frame(Year=xmonthly$Group.1,
                    Month=xmonthly$Group.2,
                    Day=rep(NA,nrow(xmonthly)),
                    Hour=rep(NA,nrow(xmonthly)),
                    Minute=rep(NA,nrow(xmonthly)),
                    Value=round(xmonthly$x,1))
  ## Write monthly file
#  out <- out[which(!is.na(out$Value)), ]
#  if (nrow(out) > 0) {
    write_sef(out, outpath, meta["var"], meta["id"], meta["name"], 
              meta["lat"], meta["lon"], meta["alt"], meta["source"], meta["link"], 
              meta["units"], "mean", period="month", meta["meta"], note="monthly", keep_na=TRUE)
#  }
}

workpath <- getwd()
setwd(outpath)
file.rename(list.files(), gsub("NA","",list.files()))
setwd(workpath)
