### Plot the Craddock's test
### The first line of the data files must contain the station coordinates (lon lat)
### The first column of the data files must contain the years

library(geosphere)
library(dataresqc)
library(zoo)
library(dplyr)
library(tidyr)
library(purrr)

# Input
#inpath <- "~/Palatina/8_monthly-series/p_hom_ref/"
#outpath <- "~/Palatina/craddock_plots/"
#candidate <- "~/Palatina/8_monthly-series/p/PALAEO-RA_Palatine-Society_Andechs_178101-179212_p_monthly_hom.tsv"
refpattern <- ".tsv"
columns <-  2:13 # indices of columns to use
mincorr <- 0.5 # minimum correlation
minlength <- 1 # minimum number of synchronous non-missing years
maxdist <- 500 # maximum distance in Km
year1 <- 1781 # start year
year2 <- 1792 # end year
ignore <- c() # vector of years to ignore
ko <- NA # ignore filenames that contain this string
ratio <- FALSE # true for PRCP

Year <- rep(1781:1792,each=12)
Month <- rep(1:12,12)
Date <- data.frame(Year,Month)
#setwd("~/Palatina/8_monthly-series/p_hom_ref/")

# Work on data
infile <- rev(strsplit(candidate, "[/]")[[1]])[1]
SEF <- read_sef(file = candidate, all = FALSE); summary(SEF$Value)
cand <- SEF[,c(2:3,5)]
cand <- merge(data.frame(Date),cand,all.x=TRUE) # ONLY IF nrow(cand) < 144
cand <- cand[,c(1,3)] %>% group_by(Year) %>% mutate(Order = seq_along(Year)) %>% spread(key = Order, value = Value)
cand <- as.data.frame(cand)
references <- list.files(inpath, pattern = refpattern, full.names = F)
names(references) <- sub("PALAEO-RA_Palatine-Society_", "", references)
#if (!is.na(ko)) {
#  ko <- grep(ko, names(references))
#  references <- references[-ko]
#}
if (candidate %in% references) references <- references[-match(infile, references)]
cand[which(cand[,1] %in% ignore), columns] <- NA
cand <- cand[, c(1, columns)]
y1 <- max(min(cand[,1]), year1)
y2 <- min(max(cand[,1]), year2)
years <- y1:y2
cand <- cand[which(cand[,1] %in% years), ]
cand_coords <- t(read.table(candidate, sep="\t", nrows=2, skip=3)[,2])
#cand_coords <- as.numeric(read_meta(candidate, parameter=c("lon","lat","alt")))
colnames(cand_coords) <- c("V1","V2")
r <- array(dim = length(references))
d <- array(dim = length(references), data=0) 
Craddock <- matrix(nrow = length(years) * length(columns), ncol = length(references))
colnames(Craddock) <- 1:length(references)
nrefs <- 0
for (iref in 1:length(references)) {
  sef1 <- read_sef(references[iref], all = FALSE)
  sef2 <- sef1[,c(2:3,5)]
  if (nrow(sef2) < 144) {
    sef3 <- merge(data.frame(Date),sef2,all.x=TRUE)
    xref <- sef3[,c(1,3)] %>% group_by(Year) %>% mutate(Order = seq_along(Year)) %>% spread(key = Order, value = Value)
  } else if (nrow(sef2) == 144) {
    xref <- sef2[,c(1,3)] %>% group_by(Year) %>% mutate(Order = seq_along(Year)) %>% spread(key = Order, value = Value)
  }
  xref <- as.data.frame(xref)
  yearsref <- xref[,1]
  xref <- xref[which(yearsref %in% years), columns]
  xrefsums <- c(t(xref), recursive = TRUE) - colMeans(xref,na.rm=TRUE)
  xref <- c(t(xref), recursive = TRUE)
  xcand <- cand[which(years %in% yearsref), 2:(length(columns)+1)]
  xcandsums <- c(t(xcand), recursive = TRUE) - colMeans(xcand,na.rm=TRUE)
  xcand <- c(t(xcand), recursive = TRUE)
  n <- ifelse(length(xcandsums) > 0, sum(!is.na(xcandsums - xrefsums)), 0)
  if (n >= minlength) {
    n <- length(xcand)
    r[iref] <- cor(xcandsums, xrefsums, use = "pairwise.complete.obs")
    if (r[iref] >= mincorr) {
      ref_coords <- t(read.table(references[iref], sep="\t", nrows=2, skip=3)[,2])
      d[iref] <- distGeo(cand_coords, ref_coords) / 1000
      if (d[iref] <= maxdist) {
        nrefs <- nrefs + 1
        colnames(Craddock)[nrefs] <- paste(names(references)[iref], round(r[iref],2), round(d[iref],0), sep=" ; ")
        xcand[which(is.na(xref))] <- NA
        xref[which(is.na(xcand))] <- NA
        if (ratio) {
          rref <- (mean(xcand, na.rm = TRUE) / mean(xref, na.rm = TRUE)) * xref
        } else {
          rref <- (mean(xcand, na.rm = TRUE) - mean(xref, na.rm = TRUE)) + xref
        }
        row1 <- (which(years == max(years[1],yearsref[1])) - 1) * length(columns) + 1
        Craddock[row1, nrefs] <- 0
        last_nomiss <- 1
        for (i in 2:n) {
          increment <- xcand[i] - rref[i]
          if (!is.na(increment)) {
            Craddock[i+row1-1, nrefs] <- Craddock[last_nomiss+row1-1, nrefs] + increment
            last_nomiss <- i
          }
        }
      }
    }
  }
}



# Plot
if (nrefs > 0) {
  outfile <- paste(outpath, infile, sep = "/")
  n <- dim(Craddock)[1]
  cols <- rainbow(nrefs)
  if (refpattern == ".tsv") {
    ylims <- c(-100, 100)
  } else if (refpattern == ".tsv") {
    ylims <- c(-2000, 2000)
  } else {
    ylims <- c(-400, 400)
  }
  pdf(paste(outfile, ".pdf", sep = ""), width = 7, height = 5)
  par(mar=c(3,3,1,1))
  #  plot(1:n, rep(NA, n), xlab = "", ylab = "Cumulative difference", ylim = c(min(Craddock, na.rm = TRUE), max(Craddock, na.rm = TRUE)), xaxt = "n", cex.axis = 1, cex.lab = 1, mgp = c(1.5,0.5,0))
  plot(1:n, rep(NA, n), xlab = "", ylab = "Cumulative difference", ylim = ylims, xaxt = "n", cex.axis = 1, cex.lab = 1, mgp = c(1.75,0.5,0))
  grid(lty=2, lwd=1, nx=NA, ny=NULL, col="darkgrey")
  axis(1, seq(1,n-length(columns)+1,by=5*length(columns)), seq(y1,y2,by=5), cex.axis=1, mgp=c(1,0.5,0))
  abline(v=seq(1,n-length(columns)+1,by=length(columns)), col="lightgrey", lty=2)
  abline(v=seq(1,n-length(columns)+1,by=5*length(columns)), col="darkgrey", lty=2)
  mtext("Year", side=1, line=1.75, cex=1)
  for (iref in 1:nrefs) {
    lines(1:n, Craddock[, iref], col = cols[iref], lwd = 1)
  }
  # breakpoints <- c(1788.65)
  # abline(v=c(length(columns)*(breakpoints-1781)+1), lwd=2.5, lty=2, col="darkgrey")
  legend("topright", legend = colnames(Craddock)[1:nrefs], lwd = 1, col = cols, cex = 0.5)
  dev.off()
  system(paste("evince ", outfile, ".pdf", sep = ""))
} else {
  print("No suitable reference series available", quote = FALSE)
}

