### FREEZING POINT CORRECTION
### applied to temperature SEF files in 5_QCed folder


library(dataresqc)
library(stringr)

setwd("~/Palatina/5_QCed/")
files <- list.files()
codes <- c("AND", "BER", "BOL", "BRU", "BUD", "CAM", "COP", "DEL", "HAG", "DIJ", "DUS", "EDS",
           "ERF", "GEN", "GDH", "GTN", "GOT", "HOH", "ING", "ROC", "MAN", "MAR", "MID", "MOS",
           "MUN", "PAD", "PRA", "PYS", "REG", "ROM", "SPY", "STO", "PET", "ZEN", "TEG", "WUR", "ZAG")
for (i in 1:length(codes)) {
  setwd(paste0("~/Palatina/5_QCed/", files[i]))
  if (!grepl("CAM|DIJ|EDS|GDH|MID|MOS|PYS|STO|PET", codes[i])) {
    cod <- codes[i]
    for (k in 1:length(list.files(pattern="_ta") %>% stringr::str_subset(., "txt", negate = TRUE))) {
      sef <- read_sef(list.files(pattern="_ta")[k], all=T)
      meta <- data.frame(t(read_meta(list.files(pattern="_ta")[k])))
      out <- assign(cod, sef)
    
# rising freezing point correction
out$Value[grepl("1781",out$Year)] <- out$Value[grepl("1781",out$Year)] - 0.1
out$Value[grepl("1782",out$Year)] <- out$Value[grepl("1782",out$Year)] - 0.2
out$Value[grepl("1783",out$Year)] <- out$Value[grepl("1783",out$Year)] - 0.3
out$Value[grepl("1784",out$Year)] <- out$Value[grepl("1784",out$Year)] - 0.4
out$Value[grepl("1785",out$Year)] <- out$Value[grepl("1785",out$Year)] - 0.5
out$Value[grepl("1786|87|88|89|90|91|92",out$Year)] <- out$Value[grepl("1786|87|88|89|90|91|92",out$Year)] - 0.6

# write files
write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = getwd(),
          variable = "ta", cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = gsub(" .*", "", meta$source), link = "NA",
          units = "C", stat = meta$stat,
          metaHead = paste0(meta$meta, "|FPC=Y"),
          meta = out$Meta, period = out$Period,
          keep_na = FALSE,
          note = gsub(".*_ta|.*_ta_|.tsv", "",list.files(pattern="_ta")[k]))
  }
 } else {
  for (k in 1:length(list.files(pattern="_ta") %>% stringr::str_subset(., "txt", negate = TRUE))) {
    sef <- read_sef(list.files(pattern="_ta")[k], all=T)
    meta <- data.frame(t(read_meta(list.files(pattern="_ta")[k])))
    out <- assign(cod, sef)
    
    # write files
    write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")],
              outpath = getwd(),
              variable = "ta", cod = meta$id,
              nam = meta$name, lat = meta$lat,
              lon = meta$lon, alt = meta$alt,
              sou = gsub(" .*", "", meta$source), link = "NA",
              units = "C", stat = meta$stat,
              metaHead = paste0(meta$meta, "|FPC=N"),
              meta = out$Meta, period = out$Period,
              keep_na = FALSE,
              note = gsub(".*_ta|.*_ta_|.tsv", "",list.files(pattern="_ta")[k]))
  }
 }
}
# rm(out)