### HOMOGENISATION
# following an evaluation of the Craddock plots,
# this script reads files with breakpoints, compares them to reference series to calculate the amount
# by which the candidate series needs to be adjusted either before, after, or in-between breakpoints.
# The corrected monthly series can be written and the Craddock test performed again to check improvements.
# Ideally the correction is applied to the subdaily/daily data and then the means re-calculated (not in this script).


library(dataresqc)

setwd("") # path to folder containing p monthly series (not corrected)
codes <- c("AND", "BER", "BOL", "BRU", "BUD", "COP", "DEL", "HAG", "DIJ", "DUS", "EDS",
           "ERF", "GEN", "GDH", "GTN", "GOT", "HOH", "ING", "ROC", "MAN", "MAR", "MID", "MOS",
           "MUN", "PAD", "PRA", "PYS", "REG", "ROM", "SPY", "STO", "PET", "ZEN", "TEG", "WUR", "ZAG")
Year <- rep(1781:1792, each=12); Month <- rep(1:12, 12); mask <- data.frame(Year, Month)
out <- list()
for (i in 1:length(codes)) {
  if (!grepl("MAN|PAD|MUN|ZAG", codes[i])) {
    cod <- codes[i]
    sef <- read_sef(list.files(pattern=".tsv")[i], all=T)
    out <- assign(cod, merge(mask, sef, all.x=T))
  } else {
    cod <- codes[i]
    sef <- read_sef(list.files(pattern=".tsv")[i], all=T)
    out <- assign(cod, sef)
  }
}
#rm(out)



# BOLOGNA, breakpoints at 1) Dec 1783 and 2) Dec 1784
# stations used for comparison: PAD, REG, MUN
prim <- mean(BOL$Value[which(BOL$Year<1784)] - (PAD$Value[which(BOL$Year<1784)]+MUN$Value[which(BOL$Year<1784)]+REG$Value[which(BOL$Year<1784)])/3, na.rm=T)
sec <- mean(BOL$Value[grepl("84",BOL$Year)] - (PAD$Value[grepl("84",BOL$Year)]+MUN$Value[grepl("84",BOL$Year)]+REG$Value[grepl("84",BOL$Year)])/3, na.rm=T)
ter <- mean(BOL$Value[which(BOL$Year>=1786)] - (PAD$Value[which(BOL$Year>=1786)]+MUN$Value[which(BOL$Year>=1786)]+REG$Value[which(BOL$Year>=1786)])/3, na.rm=T)

ter - prim #+8.2 before 1st breakpoint
ter - sec  #+3.7 in-between 1st and 2nd breakpoint

# apply changes
BOL$Value[which(BOL$Year<1784)] <- BOL$Value[which(BOL$Year<1784)] + 8.2
BOL$Value[grepl("84",BOL$Year)] <- BOL$Value[grepl("84",BOL$Year)] + 3.7

# read meta
meta <- data.frame(t(read_meta(""))) #read BOL meta

write_sef(BOL[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = "", #insert outpath
          variable = meta$var, cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = "PALAEO-RA", link = "NA",
          units = meta$units, stat = meta$stat,
          metaHead = meta$meta, period = "month", #change period accordingly
          keep_na = T, note = "monthly_hom") #change note accordingly



# ROME, breakpoints at 1) Dec 1783 and 2) Oct 1788
# stations used for comparison: PAD, BOL_hom
prim <- mean(ROM$Value[which(ROM$Year<=1783)] - (PAD$Value[which(ROM$Year<=1783)]+BOL$Value[which(BOL$Year<=1783)])/2, na.rm=T)
sec <- mean(ROM$Value[37:93] - (PAD$Value[37:93]+BOL$Value[37:93])/2, na.rm=T) #Jan84 to Sep88
ter <- mean(ROM$Value[94:144] - (PAD$Value[94:144]+BOL$Value[94:144])/2, na.rm=T) #Oct88 to end of series

sec - prim #+6 before first breakpoint
sec - ter #+3.8 from 2nd breakpoint to end of series

# apply changes
ROM$Value[which(ROM$Year<=1783)] <- ROM$Value[which(ROM$Year<=1783)] + 6
ROM$Value[94:144] <- ROM$Value[94:144] + 3.8

# read meta
meta <- data.frame(t(read_meta(""))) #read ROM meta

write_sef(ROM[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = "", #insert outpath
          variable = meta$var, cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = "PALAEO-RA", link = "NA",
          units = meta$units, stat = meta$stat,
          metaHead = meta$meta, period = "month", #change period accordingly
          keep_na = T, note = "monthly_hom") #change note accordingly



# PRAGUE, breakpoint at 1) Jul 1784
# stations used for comparison: ZAG, REG, BER, MUN
prim <- mean(PRA$Value[1:43] - (REG$Value[1:43]+ZAG$Value[1:43]+BER$Value[1:43]+MUN$Value[1:43]+TEG$Value[1:43])/5, na.rm=T) #begin to Jul84
sec <- mean(PRA$Value[44:144] - (REG$Value[44:144]+ZAG$Value[44:144]+BER$Value[44:144]+MUN$Value[44:144]+TEG$Value[44:144])/5, na.rm=T)

sec - prim #+4 before first breakpoint

# apply changes
PRA$Value[1:43] <- PRA$Value[1:43] + 4

# read meta
meta <- data.frame(t(read_meta(""))) #read PRA meta

write_sef(PRA[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = "",
          variable = meta$var, cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = "PALAEO-RA", link = "NA",
          units = meta$units, stat = meta$stat,
          metaHead = meta$meta, period = "month", #change period accordingly
          keep_na = T, note = "monthly_hom") #change note accordingly



# ANDECHS, breakpoints at 1) Aug 1786 and 2) Mar 1787
# stations used for comparison: TEG, HOH, MUN, REG, PAD
prim <- mean(AND$Value[1:18] - (TEG$Value[1:18]+HOH$Value[1:18]+MUN$Value[1:18]+REG$Value[1:18]+PAD$Value[1:18]+MAN$Value[1:18])/6, na.rm=T)
#sec <- mean(AND$Value[68:74] - (TEG$Value[68:74]+HOH$Value[68:74]+MUN$Value[68:74]+REG$Value[68:74]+PAD$Value[68:74]+MAN$Value[68:74])/6, na.rm=T)
ter <- mean(AND$Value[19:144] - (TEG$Value[19:144]+HOH$Value[19:144]+MUN$Value[19:144]+REG$Value[19:144]+
                                   PAD$Value[19:144]+MAN$Value[19:144])/6, na.rm=T)

prim - ter

# apply changes
AND$Value[1:18] <- AND$Value[1:18] - 1.6

# read meta
meta <- data.frame(t(read_meta(""))) #read AND meta

write_sef(AND[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = "", #insert outpath
          variable = meta$var, cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = "PALAEO-RA", link = "NA",
          units = meta$units, stat = meta$stat,
          metaHead = meta$meta, period = "month", #change period accordingly
          keep_na = T, note = "monthly_hom") #change note accordingly




# ERFURT, breakpoints at 1) Oct 1783 and 2) Jan 1787
# stations used for comparison: MAN, BER, REG
prim <- mean(ERF$Value[which(ERF$Year<1783)] - (MAN$Value[which(ERF$Year<1783)]+BER$Value[which(ERF$Year<1783)]+
                                                  REG$Value[which(ERF$Year<1783)])/3, na.rm=T)
sec <- mean(ERF$Value[25:72] - (MAN$Value[25:72]+BER$Value[25:72]+REG$Value[25:72])/3, na.rm=T)
ter <- mean(ERF$Value[73:144] - (MAN$Value[73:144]+BER$Value[73:144]+REG$Value[73:144])/3, na.rm=T) #Feb87 to end

sec - prim #+3.1 before first breakpoint
sec - ter  #+2.4 from second breakpoint to end of series

# apply changes
ERF$Value[which(ERF$Year<1783)] <- ERF$Value[which(ERF$Year<1783)] + 3.1
ERF$Value[73:144] <- ERF$Value[73:144] + 2.4

# read meta
meta <- data.frame(t(read_meta(""))) #read ERF meta

write_sef(ERF[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = "", #insert outpath
          variable = meta$var, cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = "PALAEO-RA", link = "NA",
          units = meta$units, stat = meta$stat,
          metaHead = meta$meta, period = "month", #change period accordingly
          keep_na = T, note = "monthly_hom") #change note accordingly



# BRUSSELS, breakpoints at 1) Nov 1783, 2) Apr 1786 and 3) Dec 1787
# stations used for comparison: MID, MAN
prim <- mean(BRU$Value[1:34] - (MAN$Value[1:34]+MID$Value[1:34])/2, na.rm=T)
sec <- mean(BRU$Value[35:64] - (MAN$Value[35:64]+MID$Value[35:64]+DEL$Value[35:64])/3, na.rm=T)
ter <- mean(BRU$Value[65:84] - (MAN$Value[65:84]+MID$Value[65:84]+DEL$Value[65:84])/3, na.rm=T)
quar <- mean(BRU$Value[85:144] - (MAN$Value[85:144]+MID$Value[85:144])/2, na.rm=T)

(prim + quar)/2 - sec
(prim + quar)/3 - ter #+5 in-between 2nd and 3rd breakpoints

#BRU$Value[38:41] <- BRU$Value[38:41] - 4 #these months should have lower values
BRU$Value[65:84] <- BRU$Value[65:84] + 5

# read meta
meta <- data.frame(t(read_meta(""))) #read BRU meta

write_sef(BRU[,c("Year","Month","Day","Hour","Minute","Value")],
          outpath = "", #insert outpath
          variable = meta$var, cod = meta$id,
          nam = meta$name, lat = meta$lat,
          lon = meta$lon, alt = meta$alt,
          sou = "PALAEO-RA", link = "NA",
          units = meta$units, stat = meta$stat,
          metaHead = meta$meta, period = "month", #change period accordingly
          keep_na = T, note = "monthly_hom") #change note accordingly




plot(); lines() # for visualisation
