library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)
library(reshape2)

##################################
###          BRUSSELS
##################################


#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Brussels")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily"), ]

#files
files <- c("Palatine-Society_T3_BE_Brussels_1782-1783_subdaily.xlsx",
           "Palatine-Society_T3_BE_Brussels_1784-1786_subdaily.xlsx",
           "Palatine-Society_T3_BE_Brussels_1788_subdaily.xlsx",
           "Palatine-Society_T3_BE_Brussels_1789-1792_subdaily.xlsx")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(2,4)) {
  template[[i]] <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Brussels/",files[i]),
                                              sheetIndex=1, startRow=5, header=FALSE)[,1:8])
}
template <- Reduce(rbind, template)
xmean <- round(aggregate(template[,c(7:8)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Brussels/",files[i]),
                        sheetIndex=1, startRow=5, header=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (grepl("82", files[i])) {
    template[,5] <- na.locf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    template$atb <- NA
    for (k in c(1:12)) {
      template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
            template$X8[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         template$atb*1.25), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6], 
                              "Pin|atb=", template$atb, "R (extrapolated from ta)")
  } else if (grepl("84", files[i])) {
    # 1784 does not have atb, but 1785 and 1786 do, so they are processed separately:
    template[,5] <- na.locf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    template$atb <- NA
    for (k in c(1:12)) {
      template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
            template$X8[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
    p1 <- round(convert_pressure(p_in[which(template[,1]==1784)], 27.07, 
                                 as.numeric(metadata$Latitude), 
                                 as.numeric(metadata$Altitude),
                                 template$atb[which(template[,1]==1784)]*1.25), 1)
    p1_meta <- paste0(meta_time[which(template[,1]==1784)], "|orig=", template[,5][which(template[,1]==1784)], 
                      ".", template[,6][which(template[,1]==1784)], "Pin|atb=", 
                      template$atb[which(template[,1]==1784)], "R (extrapolated from ta)")
    p2 <- round(convert_pressure(p_in[which(template[,1]!=1784)], 27.07, 
                                 as.numeric(metadata$Latitude), 
                                 as.numeric(metadata$Altitude),
                                 template[,7][which(template[,1]!=1784)]*1.25), 1)
    p2_meta <- paste0(meta_time[which(template[,1]!=1784)], "|orig=", template[,5][which(template[,1]!=1784)], 
                      ".", template[,6][which(template[,1]!=1784)], "Pin|atb=", 
                      template[,7][which(template[,1]!=1784)], "R")
    out[[i]]$p <- c(p1, p2)
    out[[i]]$p_meta <- c(p1_meta, p2_meta)
  } else if (grepl("89", files[i])) {
    template[,5] <- na.locf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         as.numeric(as.character(template[,7]))*1.25), 
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                      template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (!grepl("88", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  if (grepl("82", files[i])) {
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(as.character(template$X12), directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template$X12)
  } else if (!grepl("82", files[i])) {
    j <- ifelse(!grepl("88", files[i]), "X13", 5)
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(as.character(template[,j]), directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])
  }

  #wind force
  if (grepl("82", files[i])) {
    template$X13[template$X13 == "0"] <- NA
    out[[i]]$w <- as.integer(as.character(template$X13))
    out[[i]]$w_meta <- paste0(meta_time, "|orig=", template$X13)
  } else if (!grepl("82", files[i])) {
    j <- ifelse(!grepl("88", files[i]), "X14", 6)
    template[,j][template[,j] == "0"] <- NA
    out[[i]]$w <- as.integer(as.character(template[,j]))
    out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j])
  }

  #precipitation
  if (!grepl("82|88", files[i])) {
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template$X15, "lin.p.12")
    template$X15 <- colsplit(as.character(template$X15),",",names=c("lines12","lines12*12","lines144*12"))
    template$X15[is.na(template$X15)] <- 0
    template[which(time!=14),"X15"] <- NA
    out[[i]]$rr <- round((template$X15$lines12/12 + template$X15$`lines12*12`/144 +
                            template$X15$`lines144*12`/(144*12)) * 27.07, 1)
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }

  #evaporation
  if (!grepl("82|88", files[i])) {
    out[[i]]$eee_meta <- paste0(meta_time, "|orig=", template$X16, "lin.p.12")
    template$X16 <- colsplit(as.character(template$X16),",",names=c("lines12","lines12*12","lines144*12"))
    template$X16[is.na(template$X16)] <- 0
    template[which(time!=14),"X16"] <- NA
    out[[i]]$eee <- round((template$X16$lines12/12 + template$X16$`lines12*12`/144 +
                            template$X16$`lines144*12`/(144*12)) * 27.07, 1)
  } else {
    out[[i]]$eee <- NA
    out[[i]]$eee_meta <- ""
  }
}


#merge dataframes
out <- Reduce(rbind, out)
out$rr[c(2191:3844,8228:10420)] <- NA
out$eee[-c(6038:8227)] <- NA
#plot(out$rr);plot(out$eee)


#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$eee_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily"), ]
#files
files <- c("Palatine-Society_T3_BE_Brussels_1788_daily.xlsx")

#read data
template <- read.xlsx(paste0("../../3_Revised/Brussels/",files),
                      sheetIndex=1, startRow=5, header=FALSE)

#time
hour <- rep(NA, length(template[,1]))
minute <- rep(NA, length(template[,1]))
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]),
                  hour = hour, minute = minute)

#pressure
template[,4] <- na.locf(template[,4])
p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
        template$X6[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
out$p <- round(convert_pressure(p_in, 27.07, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude),
                                template$atb*1.25), 1)
out$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                     "Pin", "|atb=", template$atb, "R (extrapolated from ta)")

#temperature
out$ta <- round(as.numeric(as.character(template[,6])) * 1.25, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,6], "R")


#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")



### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly"), ]
#files
files <- c("Palatine-Society_T3_BE_Brussels_1787_monthly.xlsx")

#read data
template <- read.xlsx(paste0("../../3_Revised/Brussels/",files),
                      sheetIndex=1, startRow=6, header=FALSE)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

template$atb <- NA
#pressure
p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$pmax <- round(convert_pressure(p_in_max, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
out$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin")
#out$day <- template[,5]
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$pmin <- round(convert_pressure(p_in_min, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
out$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin")
#out$day <- template[,8]
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
        template$X6[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$pmed <- round(convert_pressure(p_in_med, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   template$atb*1.25), 1)
#out$day <- rep(NA, 12)
out$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin", "|atb=", template$atb, "R")
write_sef(out[,c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#temperature
out$tmax <- round(template[,11]*1.25, 1)
out$tmax_meta <- paste0("orig=", template[,11], "R")
#out$day <- template[,12]
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
out$tmin <- round(template[,13]*1.25, 1)
out$tmin_meta <- paste0("orig=", template[,13], "R")
#out$day <- template[,14]
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
out$tmed <- round(template[,15]*1.25, 1)
out$tmed_meta <- paste0("orig=", template[,15], "R")
#out$day <- rep(NA, 12)
write_sef(out[,c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#precipitation
out$rr_meta <- paste0("orig=", template[,19], "lin.p.12")
template$X19 <- colsplit(as.character(template$X19),",",names=c("lines12","lines12*12","lines144*12"))
out$rr <- round((template$X19$lines12/12 + template$X19$`lines12*12`/144 +
                   template$X19$`lines144*12`/(144*12)) * 27.07, 1)
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$rr_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")

#evaporation
out$eee_meta <- paste0("orig=", template[,20], "lin.p.12")
template$X20 <- colsplit(as.character(template$X20),",",names=c("lines12","lines12*12","lines144*12"))
out$eee <- round((template$X20$lines12/12 + template$X20$`lines12*12`/144 +
                    template$X20$`lines144*12`/(144*12)) * 27.07, 1)
write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$eee_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
