library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          COPENHAGEN
##################################

#function to fill missing values
na.lomf <- function(x) {
  na.lomf.0 <- function(x) {
    non.na.idx <- which(!is.na(x))
    if (is.na(x[1L])) {
      non.na.idx <- c(1L, non.na.idx)
    }
    rep.int(x[non.na.idx], diff(c(non.na.idx, length(x) + 1L)))
  }
  
  dim.len <- length(dim(x))
  
  if (dim.len == 0L) {
    na.lomf.0(x)
  } else {
    apply(x, dim.len, na.lomf.0)
  }
}

#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Copenhagen")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1782"), ]

#files
files <- "Palatine-Society_T3_DK_Copenhagen_1782-1788_subdaily.xlsx"

#extrapolation of atb from ta with lm
template <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Copenhagen/",files),
                                       sheetIndex=1, startRow=5, header=FALSE))
xmean <- round(aggregate(template[,c(7:8)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Copenhagen/",files),
                        sheetIndex=1, startRow=5, header=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  template[,5] <- na.lomf(template[,5])
  p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
  out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                       as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude),
                                       as.numeric(template[,7])*1.25), 
                      1)
  i_miss <- which(is.na(as.numeric(template[,7])))
  out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                               as.numeric(metadata$Latitude), 
                                               as.numeric(metadata$Altitude)),
                              1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                            "Pin|atb=", template[,7], "R")
  out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                    template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  
  #temperature
  out[[i]]$ta <- round(as.numeric(template[,8]) * 1.25, 1)
  out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  
  #wind direction
  directions <- gsub("E", "O", directions)
  out[[i]]$dd <- round(11.25 * (match(template[,12], directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,12])
  
  #wind force
  out[[i]]$w <- as.integer(template[,13])
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,13])
  
  #precipitation
  # template[which(time!=12),14] <- NA
  template[,14] <- as.numeric(paste(template[,14]))
  template[which(is.na(template[,14])),14] <- 0
  out[[i]]$rr <- round(((template[,14]) /144) * 27.07, 1)
  out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template[,14], "lin./144")
  
  #evaporation
  template[is.na(template[,15]), 15] <- 0
  template[is.na(template[,16]), 16] <- 0
  template[,15] <- as.numeric(paste0(template[,15], ".", template[,16]))
  template[,15] <- as.numeric(template[,15])
  template[which(time!=12),15] <- NA
  out[[i]]$eee <- round((as.numeric(template[,15]) / 12) * 27.07, 1)
  out[[i]]$eee_meta <- paste0(meta_time, "|orig=", template[,15], "lin.dec.")
  
}


#merge dataframes
out <- Reduce(rbind, out)
out$rr[5479:7671][out$rr[5479:7671]==0] <- NA
out$eee[5479:7671][out$eee[5479:7671]==0] <- NA
#plot(out$eee); plot(out$rr)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "p",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$eee_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily" &
                              inventory$First.Year == "1787"), ]
#files
files <- "Palatine-Society_T3_DK_Copenhagen_1787-1788_daily.xlsx"

##loop on files
#read data
template <- read.xlsx(paste0("../../3_Revised/Copenhagen/",files),
                      sheetIndex=1, startRow=5, header=FALSE)

#time
hour <- rep(NA, 731)
minute <- rep(NA, 731)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]),
                  hour = hour, minute = minute)

#pressure
template[,4] <- na.lomf(template[,4])
p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
        template$X6[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
out$p <- round(convert_pressure(p_in, 27.07, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude),
                                template$atb*1.25), 1)
out$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                     "Pin", "|atb=", template$atb, "R (extrapolated from ta)")

#temperature
out$ta <- round(as.numeric(as.character(template[,6])) * 1.25, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,6], "R")

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
