library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          ERFURT
##################################


#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Erfurt")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1781"), ]

#files
files <- c("Palatine-Society_T3_DE_Erfurt_1781_subdaily.xls",
           "Palatine-Society_T3_DE_Erfurt_1782_subdaily.xls",
           "Palatine-Society_T3_DE_Erfurt_1783_subdaily.xls",
           "Palatine-Society_T3_DE_Erfurt_1784_subdaily.xls",
           "Palatine-Society_T3_DE_Erfurt_1785_subdaily.xlsx",
           "Palatine-Society_T3_DE_Erfurt_1786_subdaily.xls",
           "Palatine-Society_T3_DE_Erfurt_1787_subdaily.xls",
           "Palatine-Society_T3_DE_Erfurt_1788_subdaily.xlsx")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(1:2)) {
  template[[i]] <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Erfurt/",files[i]),
                                     sheetIndex=1, startRow=8, header=FALSE)[,1:8])
}
template <- Reduce(rbind,template); template[,8] <- as.numeric(as.character(template[,8]))
xmean <- round(aggregate(template[,7:8], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Erfurt/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #fill indoor temp
  if (grepl("83", files[i])) {
    template[,7] <- 10
  }
  
  #pressure
  if (grepl("81|82", files[i])) {
    template[,5] <- na.locf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         as.numeric(template[,7])*1.25),
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else if (grepl("83|84|85|86", files[i])) {
    template[,5] <- na.locf(template[,5]); template[,7] <- na.locf(template[,7])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude)),
                        1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R (already reduced)")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (!grepl("87|88", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  j <- ifelse(!grepl("87|88", files[i]), "X12", 5)
  directions <- gsub("E", "O", directions)
  template[,j] <- as.character(template[,j])
  template[,j][template[,j] == "SWS"] <- "SSW"
  template[,j][template[,j] == "NWN"] <- "NNW"
  template[,j][template[,j] == "NWW"] <- "WNW"
  out[[i]]$dd <- round(11.25 * (match(template[,j], directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])

  #wind force
  j <- ifelse(!grepl("87|88", files[i]), "X13", 6)
  out[[i]]$w <- as.integer(template[,j])
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j])
  
  #precipitation and evaporation left out because units are unclear
  
}


#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily" &
                              inventory$First.Year == "1788"), ]
#files
files <- "Palatine-Society_T3_DE_Erfurt_1788_daily.xlsx"


##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Erfurt/",files),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  hour <- rep(NA, 366)
  minute <- rep(NA, 366)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates)
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]),
                         hour = hour, minute = minute)
  
  #pressure
  if (i < 3) {
    template[,4] <- na.locf(template[,4])
    p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
    template$atb <- NA
    for (k in c(1:12)) {
      template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
            template$X6[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         template$atb*1.25), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                              "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (i < 3) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,6])) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,6], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }
}


#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")



### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1787"), ]
#files
files <- c("Palatine-Society_T3_DE_Erfurt_1787-1788_monthly.xlsx")

#read data
template <- read.xlsx(paste0("../../3_Revised/Erfurt/",files),
                      sheetIndex=1, startRow=8, header=FALSE)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

#pressure
p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$pmax <- round(convert_pressure(p_in_max, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
#out$day <- c(7,1,13,21,19,8,2,3,4,6,26,28,16,7,30,10,5,6,19,2,27,9,1,25)
out$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin")
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$pmin <- round(convert_pressure(p_in_min, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
#out$day <- c(21,13,4,29,1,5,22,25,18,26,2,6,3,24,6,4,29,28,5,14,25,16,4,26)
out$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin")
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
       template$X15[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$pmed <- round(convert_pressure(p_in_med, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   template$atb*1.25), 1)
#out$day <- rep(NA, 24)
out$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
write_sef(out[,c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#temperature
out$tmax <- round(template[,11]*1.25, 1)
#out$day <- c(30,12,30,16,10,13,30,10,23,8,9,10,6,23,31,21,27,20,13,13,5,6,4,24)
out$tmax_meta <- paste0("orig=", template[,11], "R")
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
out$tmin <- round(template[,13]*1.25, 1)
#out$day <- c(27,25,21,18,18,8,20,30,30,30,28,30,31,19,18,7,5,11,28,29,26,31,26,15)
out$tmin_meta <- paste0("orig=", template[,13], "R")
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
out$tmed <- round(template[,15]*1.25, 1)
#out$day <- rep(NA, 24)
out$tmed_meta <- paste0("orig=", template[,15], "R")
write_sef(out[,c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

# precipitation and evaporation were left out because units are unclear


#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
