library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          GOETTINGEN
##################################

#function to fill missing values
na.lomf <- function(x) {
  na.lomf.0 <- function(x) {
    non.na.idx <- which(!is.na(x))
    if (is.na(x[1L])) {
      non.na.idx <- c(1L, non.na.idx)
    }
    rep.int(x[non.na.idx], diff(c(non.na.idx, length(x) + 1L)))
  }
  
  dim.len <- length(dim(x))
  
  if (dim.len == 0L) {
    na.lomf.0(x)
  } else {
    apply(x, dim.len, na.lomf.0)
  }
}

#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Goettingen")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1783"), ]

#files
files <- c("Palatine-Society_T3_DE_Goettingen_1783_subdaily.xls",
           "Palatine-Society_T3_DE_Goettingen_1784_subdaily.xlsx",
           "Palatine-Society_T3_DE_Goettingen_1785_subdaily.xlsx",
           "Palatine-Society_T3_DE_Goettingen_1787_subdaily.xls")



##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Goettingen/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE, stringsAsFactors=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (i < 4) {
    template[,5] <- na.lomf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(as.character(metadata$Altitude)),
                                         as.numeric(template[,7])*1.25), 
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                      template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (i < 4) {
    out[[i]]$ta <- round(as.numeric(template[,8]) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }
  
  #wind direction
  if (i < 4) {
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(template[,10], directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,10])
  } else if (i == 4) {
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(template[,5], directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,5])
  }
  
  #wind force
  if (i < 4) {
    out[[i]]$w <- as.integer(template[,11])
    out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,11])
  } else if (i == 4) {
    out[[i]]$w <- as.integer(template[,6])
    out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,6])
  }
  
  #precipitation
  if (i < 4) {
    template[which(is.na(template[,12])),12] <- 0
    template[which(time!=14),12] <- NA
    out[[i]]$rr <- round((as.numeric(as.character(template[,12]))/12) * 27.07, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template[,12], "lin")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }
  
  #evaporation
  if (i < 4) {
    template[which(is.na(template[,13])),13] <- 0
    template[which(template[,13]=="glac."),13] <- 0
    template[which(time!=14),13] <- NA
    out[[i]]$eee <- round((as.numeric(as.character(template[,13]))/12) * 27.07, 1)
    out[[i]]$eee_meta <- paste0(meta_time, "|orig=", template[,13], "lin")
  } else {
    out[[i]]$eee <- NA
    out[[i]]$eee_meta <- ""
  }
}



#merge dataframes
out <- Reduce(rbind, out)


#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " | orig.unit = lines (1/12 of Paris inch)"),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " | orig.unit = lines (1/12 of Paris inch)"),
          meta = out$eee_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)


#room temperature
atb <- list()
for (i in 1:3) {
  template <- read.xlsx(paste0("../../3_Revised/Goettingen/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  atb[[i]] <- round(as.numeric((template[,7]) * 1.25), 1)
}
atb <- round(mean(Reduce(cbind, atb), na.rm = TRUE), 1) # 10.3?C
atb <- rep(10.3, 12)


### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1787"), ]

#files
files <- c("Palatine-Society_T3_DE_Goettingen_1787_monthly.xlsx")

#read data
template <- read.xlsx(paste0("../../3_Revised/Goettingen/",files[1]),
                      sheetIndex=1, startRow=8, header=FALSE)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

#pressure
p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$pmax <- round(convert_pressure(p_in_max, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   atb = atb),
                  1)
#out$day <- c(7,1,14,21,20,9,2,3,2,7,27,8)
out$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin", "|atb=", atb, "C")
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$pmin <- round(convert_pressure(p_in_min, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   atb = atb),
                  1)
#out$day <- c(27,13,4,29,1,15,22,25,18,14,3,18)
out$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin", "|atb=", atb, "C")
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$pmed <- round(convert_pressure(p_in_med, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   atb = atb),
                  1)
#out$day <- rep(NA, 12)
out$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin", "|atb=", atb, "C")
write_sef(out[,c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#temperature
out$tmax <- round(template[,11]*1.25, 1)
#out$day <- c(31,11,30,12,24,13,30,6,26,6,11,10)
out$tmax_meta <- paste0("orig=", template[,11], "R")
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
out$tmin <- round(template[,13]*1.25, 1)
#out$day <- c(16,25,21,20,1,7,1,30,30,16,29,23)
out$tmin_meta <- paste0("orig=", template[,13], "R")
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
out$tmed <- round(template[,15]*1.25, 1)
#out$day <- rep(NA, 12)
out$tmed_meta <- paste0("orig=", template[,15], "R")
write_sef(out[,c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")
out$rr <- round((as.numeric(as.character(template[,21]))/12) * 27.07, 1)
#out$day <- rep(NA, 12)
out$rr_meta <- paste0("orig=", template[,21], "lin")
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$rr_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")
out$eee <- round((as.numeric(as.character(template[,22]))/12) * 27.07, 1)
#out$day <- rep(NA, 12)
out$eee_meta <- paste0("orig=", template[,22], "lin")
write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$eee_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
