library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          REGENSBURG
##################################


#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Regensburg")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1781"), ]

#files
files <- c("Palatine-Society_T3_DE_Regensburg_1781-1786_subdaily.xlsx",
           "Palatine-Society_T3_DE_Regensburg_1787-1788_subdaily.xlsx",
           "Palatine-Society_T3_DE_Regensburg_1789-1791_subdaily.xlsx")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(1,3)) {
  template[[i]] <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Regensburg/",files[i]),
                                              sheetIndex=1, startRow=5, header=FALSE)[,1:8])
}
template <- Reduce(rbind, template)
xmean <- round(aggregate(template[,c(7:8)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Regensburg/",files[i]),
                        sheetIndex=1, startRow=5, header=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (!grepl("88", files[i])) {
    template[,5] <- na.locf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         as.numeric(as.character(template[,7]))*1.25), 
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                      template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (!grepl("88", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  if (grepl("81|88", files[i])) {
  j <- ifelse(!grepl("88", files[i]), "X12", 5)
  directions <- gsub("E", "O", directions)
  out[[i]]$dd <- round(11.25 * (match(as.character(template[,j]), directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])
  } else if (grepl("89", files[i])) {
    j <- ifelse(!grepl("88", files[i]), "X14", 5)
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(as.character(template[,j]), directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])
  }

  #wind force
  if (grepl("81|88", files[i])) {
    j <- ifelse(!grepl("88", files[i]), "X13", 6)
    template[,j][template[,j] == "0"] <- NA
    out[[i]]$w <- as.integer(as.character(template[,j]))
    out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j])
  } else if (grepl("89", files[i])) {
    j <- ifelse(!grepl("88", files[i]), "X15", 6)
    template[,j][template[,j] == "0"] <- NA
    out[[i]]$w <- as.integer(as.character(template[,j]))
    out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j])
  }

  #precipitation
  if (!grepl("88|89", files[i])) {
    template$X14[is.na(template$X14)] <- 0
    day_rr <- rep(rollapply(template$X14, 3, sum, by = 3), each=3)
    day_rr[seq(1,length(day_rr),by=3)] <- NA; day_rr[seq(2,length(day_rr),by=3)] <- NA
    out[[i]]$rr <- round((day_rr /24) * 27.07, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", day_rr, "sem.gr.")
  # } else if (grepl("89|90|91", files[i])) {
  #   template$X14 <- as.numeric(as.character(template$X14))
  #   template$X14[is.na(template$X14)] <- 0
  #   template[which(time!=14),14] <- NA
  #   out[[i]]$rr <- round((template$X14 /64 /12) * 27.07, 1)
  #   out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template$X14, "lin./64")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }
}


#merge dataframes
out <- Reduce(rbind, out)
#plot(out$rr)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily" &
                              inventory$First.Year == "1788"), ]
#files
files <- c("Palatine-Society_T3_DE_Regensburg_1788_daily.xlsx")

#read data
template <- read.xlsx(paste0("../../3_Revised/Regensburg/",files),
                      sheetIndex=1, startRow=5, header=FALSE)

#time
hour <- rep(NA, length(template[,1]))
minute <- rep(NA, length(template[,1]))
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]),
                  hour = hour, minute = minute)

#pressure
template[,4] <- na.locf(template[,4])
p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
        template$X6[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
out$p <- round(convert_pressure(p_in, 27.07, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude),
                                template$atb*1.25), 1)
out$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                     "Pin", "|atb=", template$atb, "R (extrapolated from ta)")

#temperature
out$ta <- round(as.numeric(as.character(template[,6])) * 1.25, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,6], "R")


#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")


#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
