library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          ROME
##################################


#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Rome")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1782"), ]

#files
files <- c("Palatine-Society_T3_IT_Rome_1782_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1783_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1784_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1785_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1786_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1787_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1788_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1789_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1790_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1791_subdaily.xls",
           "Palatine-Society_T3_IT_Rome_1792_subdaily.xls")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(1:5,8:11)) {
  template[[i]] <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Rome/",files[i]),
                                              sheetIndex=1, startRow=8, header=FALSE)[,1:8])
}
template <- Reduce(rbind, template)
xmean <- round(aggregate(template[,c(7:8)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Rome/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (!grepl("87|88", files[i])) {
    template[,5] <- na.locf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         as.numeric(as.character(template[,7]))*1.25), 
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                      template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (!grepl("87|88", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  j <- ifelse(!grepl("87|88", files[i]), "X12", 5)
  directions <- gsub("E", "O", directions)
  out[[i]]$dd <- round(11.25 * (match(as.character(template[,j]), directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])

  #wind force
  j <- ifelse(!grepl("87|88", files[i]), "X13", 6)
  template[,j][template[,j] == "0"] <- NA
  out[[i]]$w <- as.integer(as.character(template[,j]))
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", template[,j])

  #precipitation
  if (!grepl("87|88", files[i])) {
    template$X14 <- as.numeric(as.character(template$X14))
    template$X14[is.na(template$X14)] <- 0
    template[which(time!=14),"X14"] <- NA
    out[[i]]$rr <- round((template$X14/144/12) * 27.07, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template$X14, "p.lin.144")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }

  #evaporation
  if (grepl("82", files[i])) {
    template$X15[is.na(template$X15)] <- 0; template$X16[is.na(template$X16)] <- 0
    template[which(time!=14),c("X15","X16")] <- NA
    out[[i]]$eee <- round((template$X15 + template$X16/50)/12 * 27.07, 1)
    out[[i]]$eee_meta <- paste0(meta_time, "|orig=", template$X15, ".", template$X16, "lin.part.50")
  } else if (!grepl("82|87|88", files[i])) {
    template$X15[is.na(template$X15)] <- 0; template$X16[is.na(template$X16)] <- 0
    template[which(time!=14),c("X15","X16")] <- NA
    out[[i]]$eee <- round((template$X15 + template$X16/46)/12 * 27.07, 1)
    out[[i]]$eee_meta <- paste0(meta_time, "|orig=", template$X15, ".", template$X16, "lin.part.46")
  } else {
    out[[i]]$eee <- NA
    out[[i]]$eee_meta <- ""
  }
}


#merge dataframes
out <- Reduce(rbind, out)
#plot(out$rr); plot(out$eee)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$eee_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily" &
                              inventory$First.Year == "1787"), ]
#files
files <- c("Palatine-Society_T3_IT_Rome_1787_daily.xls",
           "Palatine-Society_T3_IT_Rome_1788_daily.xls")

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Rome/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  hour <- rep(NA, length(template[,1]))
  minute <- rep(NA, length(template[,1]))
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates)
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]),
                         hour = hour, minute = minute)
  
  #pressure
  template[,4] <- na.locf(template[,4])
  p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
  template$atb <- NA
  for (k in c(1:12)) {
    template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
          template$X8[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
  out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                       as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude),
                                       template$atb*1.25), 1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                            "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
  
  #temperature
  out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
  out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
}

#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")



### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1787"), ]
#files
files <- c("Palatine-Society_T3_IT_Rome_1787_monthly.xls",
           "Palatine-Society_T3_IT_Rome_1788_monthly.xls")

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Rome/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = rep(NA, nrow(template)), 
                         hour = rep(NA, nrow(template)),
                         minute = rep(NA, nrow(template)))
  
  #precipitation
  if (i < 3) {
    out[[i]]$rr <- round((template[,3] + template[,4]/144)/12 * 27.07, 1)
    out[[i]]$rr_meta <- paste0("orig=", template[,3], ".", template[,4], "lin.part.144")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }
  
  #evaporation
  if (i < 3) {
    out[[i]]$eee <- round((template[,5] + template[,6]/46)/12 * 27.07, 1)
    out[[i]]$eee_meta <- paste0("orig=", template[,5], ".", template[,6], "lin.part.46")
  } else {
    out[[i]]$eee <- NA
    out[[i]]$eee_meta <- ""
  }
  
  # number of rainy days
  if (grepl("87", files[i])) {
    out[[i]]$nr <- template[,7]
    out[[i]]$nr_meta <- paste0("orig=", template[,7])
  } else {
    out[[i]]$nr <- template[,9]
    out[[i]]$nr_meta <- paste0("orig=", template[,9])
  }
  
}

#merge dataframes
out <- Reduce(rbind, out)

# write SEF files
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$rr_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")

write_sef(out[,c("year","month","day","hour","minute","eee")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "eee",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$eee_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")

write_sef(out[,c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "count",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$nr_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
